# -*- coding: utf-8 -*-
##############################################################################
#
#    ValueDecision Twitter Search Module for OpenERP
#    Copyright (C) 2011 Les Green (<l.green@valuedecision.com>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import netsvc
import tools
import json
from urllib import urlopen, urlencode
from osv import osv, fields
from datetime import datetime
import urlparse

logger = netsvc.Logger()

class TwitterSearchServer(osv.osv):
    _name = 'twitter.search.server'
    _description = "Twitter Search Server"

    _columns = {
        'name':fields.char('Name', size=256, required=True),
        'active':fields.boolean('Active', required=True),
        'url' : fields.char('Url', size=256, required=True),
        'query':fields.char('Query', size=256, required=True, help="Search query string: Follow these guidelines to create your query - http://search.twitter.com/operators"),
        'object_id': fields.many2one('ir.model', "Model", help="OpenObject Model. Generates a record of this model.\nSelect Object with message_new attrbutes."),
        'priority': fields.integer('Server Priority', help="Priority between 0 to 10, select define the order of Processing"),
        'page_depth': fields.integer('Page Depth', help="How many pages of results to fetch (if they exist)"),
        'user_id':fields.many2one('res.users', 'User', required=False),
        'last_fetched_date': fields.datetime('Last Fetched', readonly=True)
    }
    _defaults = {
        'active': True,
        'user_id': lambda self, cr, uid, ctx: uid,
        'url': 'http://search.twitter.com/search.json',
        'page_depth': 1,
        'priority': 10 #Not currently implemented
    }
    

    def button_fetch_tweets(self, cr, uid, ids, context=None):
        self.fetch_tweets(cr, uid, ids, context=context)
        return True

    def _fetch_tweets(self, cr, uid, ids=False, context=None):
        logger.notifyChannel('twitter_search', netsvc.LOG_INFO, 'Fetching Tweets from All active searches')
        if not ids:
            ids = self.search(cr, uid, [])
        return self.fetch_tweets(cr, uid, ids, context=context)

    def fetch_tweets(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        tweetInsert = self.pool.get('twitter.search.tweets')    
        try:
            for server in self.browse(cr, uid, ids, context=context):
                user = server.user_id.id or uid
                current_depth=1;
                tweetInserts=0;
                while current_depth<=server.page_depth:
                    moreResults=server.query if current_depth==1 else tweets.get('next_page')
                    logger.notifyChannel('twitter_search', netsvc.LOG_DEBUG, 'depth:(%s/%s)'%(current_depth,server.page_depth))
                    if not moreResults or moreResults=='': break
                    preParams = {'lang': 'en', 'q':server.query }
                    if current_depth>1:
                        qs = urlparse.parse_qs(tweets.get('next_page'))
                        preParams['page']=current_depth;
                        preParams['max_id']=qs['max_id'][0]
                   
                    params = urlencode(preParams)
                    logger.notifyChannel('twitter_search', netsvc.LOG_DEBUG, 'Url:%s?%s'%(server.url, params))
                    doc = urlopen("%s?%s" % (server.url, params))
                    tweets = json.load(doc)
                
                    logger.notifyChannel('twitter_search', netsvc.LOG_DEBUG, 'Got %s Tweets to process'%len(tweets))
                    for tweet in tweets['results']:
                        # Clunky - Les2do - make this more elegant
                        if tweetInsert.search(cr,uid,[('twitter_id','=',tweet['id'])]): 
                            continue
                    
                        tweetInsert.create(cr, uid, self._getTweetValues(tweet, server.id))
                        tweetInserts+=1
                    
                        logger.notifyChannel('twitter_search', netsvc.LOG_DEBUG, 'Tweet %s inserted'%tweetInserts)
                        if server.object_id:
                            self._insertExtraObject(cr,uid,server.object_id.name,tweet,context)
                
                    current_depth+=1
                    logger.notifyChannel('twitter_search', netsvc.LOG_DEBUG, 'Updating Depth to:%s'%current_depth)
  
                self.write(cr,uid,server.id,{'last_fetched_date': datetime.now()}) 
                logger.notifyChannel('twitter_search', netsvc.LOG_INFO, 'twitter_search inserted %s tweets(s) for %s' % (tweetInserts, server.name))

            logger.notifyChannel('twitter_search',netsvc.LOG_DEBUG, 'twitter search run')
        except Exception, e:
            logger.notifyChannel("twitter_search", netsvc.LOG_WARNING, '%s' % (e))
        return True
        
    def _getTweetValues(self,tweet,serverId):
        values = {
                  'twitter_id': tweet['id'],
                  'created_at': tweet['created_at'],
                  'from_user': tweet['from_user'],
                  'from_user_id': tweet['from_user_id'],
                  'geo': tweet['geo'],
                  'iso_language_code': tweet['iso_language_code'],
                  'profile_image_url': tweet['profile_image_url'],
                  'source': tweet['source'],
                  'text': tweet['text'],
                  'server_search_id': serverId  
                  }
        if 'to_user_id' in tweet:
            values['to_user_id']=tweet['to_user_id']
        
        if 'to_user' in tweet:
            values['to_user']=tweet['to_user']

        return values
        
    def _insertExtraObject(self,cr,uid,object_name,tweet,context=None):
        #Currently crm.lead and crm.helpdesk are supported
        objectInsert = self.pool.get(self._getModel(cr, uid, object_name))
        msg = {
               'name': 'Tweet from %s (id:%s)' %  (tweet['from_user'],tweet['id']),
               'contact_name': tweet['from_user'],
               'description': ''.join(['From: @',tweet['from_user'],'\nTweet: ',tweet['text']])
        }
        objectInsert.create(cr,uid,msg,context)
    
    def _getModel(self,cr,uid,object_name):
        irModel = self.pool.get('ir.model')
        modelId = irModel.search(cr,uid,[('name','=',object_name)])
        model = irModel.read(cr,uid,modelId,['model'])
        return model[0]['model'] 
      
TwitterSearchServer()

class TwitterSearchTweets(osv.osv):
    _name = 'twitter.search.tweets'
    _description = "Store of the Tweets"
    
    _columns = {
        'twitter_id':fields.char('ID', size=256, required=True),
        'created_at':fields.datetime('Created at'),
        'from_user': fields.char('From User',size=256),
        'from_user_id': fields.integer('TwitterFromUserId'),
        'geo':fields.char('Geo',size=256),
        'iso_language_code': fields.char('ISO code', size=5),
        'profile_image_url': fields.char('ProfileImageUrl',size=256),
        'source': fields.char('Source',size=256),
        'text': fields.char('Text',size=256),  
        'to_user': fields.char('To User',size=256),
        'to_user_id': fields.integer('TwitterToUserId'),
        'server_search_id': fields.many2one('twitter.search.server','Search Server Id', required=True)
    }
    _sql_constraints = [('twitter_id','unique(twitter_id)','Tweet ID must be unique')]

TwitterSearchTweets()